==> peekabot-0.6.0.tar.bz2: peekabot-0.6.0/src/actions/SetRotation.cc <==
/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>

#include "SetRotation.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#endif


using namespace peekabot;


SetRotation::SetRotation(
    ObjectID target,
    real_unit yaw,
    real_unit roll,
    real_unit pitch,
    CoordinateSystem coord_sys) throw()
    : m_target(target),
      m_yaw(yaw),
      m_roll(roll),
      m_pitch(pitch),
      m_coord_sys(coord_sys)
{
}


SetRotation::SetRotation() throw()
{
}


SetRotation::SetRotation(const SetRotation &action) throw()
    : m_target(action.m_target),
      m_yaw(action.m_yaw),
      m_roll(action.m_roll),
      m_pitch(action.m_pitch),
      m_coord_sys(action.m_coord_sys)
{
}


SetRotation::~SetRotation() throw()
{
}


Action *SetRotation::clone() const
{
    return new SetRotation(*this);
}


void SetRotation::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error(
            "Target object not found");

    // Check for infinity and NaN
    if( !boost::math::isfinite(m_yaw) ||
        !boost::math::isfinite(m_roll) ||
        !boost::math::isfinite(m_pitch) )
        throw std::logic_error(
            "Failed to set rotations: angles contain infinity or NaN");

    Matrix4r rot;

    switch( m_coord_sys )
    {
        case LOCAL_COORDINATES:
            rot = (Matrix4ru::rotate_z(m_yaw) *
                   Matrix4ru::rotate_y(m_roll) *
                   Matrix4ru::rotate_x(m_pitch));
            break;

        case PARENT_COORDINATES:
            rot = (Matrix4ru::rotate_z(m_yaw) *
                   Matrix4ru::rotate_y(m_roll) *
                   Matrix4ru::rotate_x(m_pitch));
            break;

        case WORLD_COORDINATES:
            // Rotate, then translate and at last convert to parent coordinates
            rot = (ptr->get_parent_mtow().inverse() *
                   Matrix4ru::rotate_z(m_yaw) *
                   Matrix4ru::rotate_y(m_roll) *
                   Matrix4ru::rotate_x(m_pitch));
            break;

        default:
            throw std::runtime_error(
                "Failed to set rotation: unsupported coordinate system");
            break;
    }

    rot(3) = ptr->get_position();
    ptr->set_transformation(rot);
#endif
}


void SetRotation::save(SerializationInterface &ar) const
{
    ar << m_target
       << m_yaw << m_roll << m_pitch
       << m_coord_sys;
}


void SetRotation::load(DeserializationInterface &ar)
{
    ar >> m_target
       >> m_yaw >> m_roll >> m_pitch
       >> m_coord_sys;
}

==> peekabot-0.7.2.tar.bz2: peekabot-0.7.2/src/actions/SetRotation.cc <==
/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>

#include "SetRotation.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#endif


using namespace peekabot;


SetRotation::SetRotation(
    ObjectID target,
    float yaw,
    float pitch,
    float roll,
    CoordinateSystem coord_sys) throw()
    : m_target(target),
      m_yaw(yaw),
      m_pitch(pitch),
      m_roll(roll),
      m_coord_sys(coord_sys)
{
}


SetRotation::SetRotation() throw()
{
}


SetRotation::SetRotation(const SetRotation &action) throw()
    : m_target(action.m_target),
      m_yaw(action.m_yaw),
      m_pitch(action.m_pitch),
      m_roll(action.m_roll),
      m_coord_sys(action.m_coord_sys)
{
}


SetRotation::~SetRotation() throw()
{
}


Action *SetRotation::clone() const
{
    return new SetRotation(*this);
}


void SetRotation::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error(
            "Target object not found");

    // Check for infinity and NaN
    if( !boost::math::isfinite(m_yaw) ||
        !boost::math::isfinite(m_pitch) ||
        !boost::math::isfinite(m_roll) )
        throw std::logic_error(
            "Failed to set rotations: angles contain infinity or NaN");

    Matrix4r rot;

    switch( m_coord_sys )
    {
        case LOCAL_COORDINATES:
            rot = ptr->get_transformation();
            rot(3) = Vector4f(0,0,0,1);
            rot = (Matrix4ru::rotate_x(m_roll) *
                   Matrix4ru::rotate_y(m_pitch) *
                   Matrix4ru::rotate_z(m_yaw) * rot);
            break;

        case PARENT_COORDINATES:
            rot = (Matrix4ru::rotate_x(m_roll) *
                   Matrix4ru::rotate_y(m_pitch) *
                   Matrix4ru::rotate_z(m_yaw));
            break;

        case WORLD_COORDINATES:
            // Rotate, then translate and at last convert to parent coordinates
            rot = (ptr->get_parent_mtow().inverse() *
                   Matrix4ru::rotate_x(m_roll) *
                   Matrix4ru::rotate_y(m_pitch) *
                   Matrix4ru::rotate_z(m_yaw));
            break;

        default:
            throw std::runtime_error(
                "Failed to set rotation: unsupported coordinate system");
            break;
    }

    rot(3) = ptr->get_position();
    ptr->set_transformation(rot);
#endif
}


void SetRotation::save(SerializationInterface &ar) const
{
    ar << m_target
       << m_yaw << m_roll << m_pitch
       << m_coord_sys;
}


void SetRotation::load(DeserializationInterface &ar)
{
    ar >> m_target
       >> m_yaw >> m_roll >> m_pitch
       >> m_coord_sys;
}
